package  
{
	
	import net.flashpunk.World;
	import net.flashpunk.utils.Draw;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.Sfx;
	
	/**
	 * ...
	 * @author Benn
	 */
	public class GameWorld extends World 
	{
		
		[Embed(source="../assets/audio/ld30v2.mp3")] private static const MUSIC:Class;
		
		private var Music:Sfx = new Sfx(MUSIC);
		
		private var GamePlayer : Player;
		private var CircleArray : Array = [];
		private var CurrentCircle : int = 1;
		
		private var BackgroundArcPos : Number = 1;
		
		private var EndGameTriggered : Boolean = false;
		private var KillInput : Boolean = false;
		
		private var TutText : HudText;
		private var CreditText : HudText;
		
		private var ThanksText : HudText;
		
		//camera lerp
		private var CamIsLerping : Boolean = false;
		private var CamStartLerpPosX : int;
		private var CamStartLerpPosY : int;
		private var CamLerpTimer : Number = 1.0;
		
		public function GameWorld() 
		{
			Music.loop();
			
		}
		
		override public function begin():void 
		{
			
			var newCircle : CircleWorld = new CircleWorld(1);
			CircleArray.push(newCircle);
			
			newCircle = new CircleWorld(2);
			CircleArray.push(newCircle);
			
			newCircle = new CircleWorld(3);
			CircleArray.push(newCircle);
			
			newCircle = new CircleWorld(4);
			CircleArray.push(newCircle);
			
			newCircle = new CircleWorld(5);
			CircleArray.push(newCircle);
			
			GamePlayer = new Player();
			GamePlayer.SetCurrentCircle(CircleArray[0]);
			add(GamePlayer);
			
			TutText = new HudText("Circular Logic");
			TutText.x = 330;
			TutText.y = 280;
			add(TutText);
			
			CreditText = new HudText("by Benn Powell");
			CreditText.x = 315;
			CreditText.y = 310;
			add(CreditText);
			
			super.begin();
			
		}
		
		override public function update():void 
		{
			
			
			
			if (Input.pressed(Key.M))
			{
				if (Music.playing)
				{
					Music.stop();
				}
				else
				{
					Music.loop();
				}
			}
			
			BackgroundArcPos += 50 * FP.elapsed;
			
			if (KillInput)
			{	
				super.update();
				return;
			}
			
			if (EndGameTriggered)
			{
				
				if (CamIsLerping)
				{
					
					FP.camera.x = FP.lerp(CamStartLerpPosX, (GamePlayer.x - 400), 1.0 - CamLerpTimer);
					FP.camera.y = FP.lerp(CamStartLerpPosY, (GamePlayer.y - 300), 1.0 - CamLerpTimer);
					
					CamLerpTimer -= 2 * FP.elapsed;
					
					if (CamLerpTimer < 0)
					{
						CamLerpTimer = 1.0;
						CamIsLerping = false;
					}
					
				}
				else
				{
					FP.camera.x = GamePlayer.x - 400;
					FP.camera.y = GamePlayer.y - 300;
				}
	
				if (GamePlayer.x < -2000)
				{
					
					ThanksText = new HudText("Thanks for playing!");
					ThanksText.x = FP.camera.x + 300;
					ThanksText.y = FP.camera.y + 290;
					add(ThanksText);
					
					KillInput = true;
				}
				super.update();
				return;
			}
			
			if (GamePlayer.GetMovingBetweenWorlds())
			{
				
				if (CamIsLerping)
				{
					
					FP.camera.x = FP.lerp(CamStartLerpPosX, (GamePlayer.x - 400), 1.0 - CamLerpTimer);
					FP.camera.y = FP.lerp(CamStartLerpPosY, (GamePlayer.y - 300), 1.0 - CamLerpTimer);
					
					CamLerpTimer -= 2 * FP.elapsed;
					
					if (CamLerpTimer < 0)
					{
						CamLerpTimer = 1.0;
						CamIsLerping = false;
					}
					
				}
				else
				{
					FP.camera.x = GamePlayer.x - 400;
					FP.camera.y = GamePlayer.y - 300;
				}
				
				if (GamePlayer.GetMovementComplete())
				{
					CamIsLerping = true;
					CamStartLerpPosX = FP.camera.x;
					CamStartLerpPosY = FP.camera.y;
					GamePlayer.SetCurrentCircle(CircleArray[CurrentCircle]);
					GamePlayer.Reset();
					CurrentCircle++;
				}
				
			}
			else
			{
				
				if (CamIsLerping)
				{
					FP.camera.x = FP.lerp(CamStartLerpPosX, (CircleArray[CurrentCircle-1].x - 400), 1.0 - CamLerpTimer);
					FP.camera.y = FP.lerp(CamStartLerpPosY, (CircleArray[CurrentCircle-1].y - 300), 1.0 - CamLerpTimer);
					
					CamLerpTimer -= 2 * FP.elapsed;
					
					if (CamLerpTimer < 0)
					{
						CamLerpTimer = 1.0;
						CamIsLerping = false;
					}
				}
				else
				{
					FP.camera.x = CircleArray[CurrentCircle-1].x - 400;
					FP.camera.y = CircleArray[CurrentCircle-1].y - 300;
				}

				if (GamePlayer.GetOutOfBounds())
				{
					
					if (CurrentCircle == 1)
					{					
						CamStartLerpPosX = FP.camera.x;
						CamStartLerpPosY = FP.camera.y;
						CamIsLerping = true;
						GamePlayer.SetMoveBetweenWorlds(400, 50, 400, -400, 750, -400);
					}
					else if (CurrentCircle == 2)
					{
						
						if (GamePlayer.x > 345 || GamePlayer.x < 15)
						{
							CamStartLerpPosX = FP.camera.x;
							CamStartLerpPosY = FP.camera.y;
							CamIsLerping = true;
							GamePlayer.SetMoveBetweenWorlds(1250, -400, 1500, -400, 1500, -650);
						}
						else
						{
							GamePlayer.Reset();
						}	
						
					}
					else if (CurrentCircle == 3)
					{
						
						if (GamePlayer.x > 160 && GamePlayer.x < 190)
						{
							CamStartLerpPosX = FP.camera.x;
							CamStartLerpPosY = FP.camera.y;
							CamIsLerping = true;
							GamePlayer.SetMoveBetweenWorlds(1250, -900, 1100, -900, 1100, -810);		
						}
						else
						{
							GamePlayer.Reset();
						}	
					
					}
					else if (CurrentCircle == 4)
					{
						
						CamStartLerpPosX = FP.camera.x;
						CamStartLerpPosY = FP.camera.y;
						CamIsLerping = true;
						GamePlayer.SetMoveBetweenWorlds(1052.5, -747.5, 923.75, -747.5, 795, -747.5);
						
					}
					else if (CurrentCircle == 5)
					{
						
						if (GamePlayer.x > 160 && GamePlayer.x < 190)
						{
							CamIsLerping = true;
							CamStartLerpPosX = FP.camera.x;
							CamStartLerpPosY = FP.camera.y;
							EndGameTriggered = true;
							GamePlayer.SetMovingFinal();
						}
						else
						{
							GamePlayer.Reset();
						}
						
					}
					
				}
			}
			/*
			if (Input.check(Key.I))
			{
				FP.camera.y -= 500 * FP.elapsed;
			}
			else if (Input.check(Key.K))
			{
				FP.camera.y += 500 * FP.elapsed;
			}
			
			if (Input.check(Key.J))
			{
				FP.camera.x -= 500 * FP.elapsed;
			}
			else if (Input.check(Key.L))
			{
				FP.camera.x += 500 * FP.elapsed;
			}
			*/
			super.update();
		}
	
		private function draw_arc( center_x : int, center_y : int, radius : int, angle_from : Number, angle_to : Number, precision : Number, color : uint, thickness : Number) : void
		{
			
            var angle_diff : Number = angle_to - angle_from;
            var steps : int = Math.round(angle_diff * precision);
            var angle : Number = angle_from;
			
            var startx : Number = center_x + radius * Math.cos( angle * FP.RAD );
            var starty : Number = center_y + radius * Math.sin( angle * FP.RAD );
			
			/*
			trace(startx);
			trace(starty);
			*/
			
            for (var i:int = 1; i <= steps; i++) 
			{
				
				angle = angle_from + angle_diff / steps * i;
				
				var endx : Number = center_x + radius * Math.cos(angle * FP.RAD);
				var endy : Number = center_y + radius * Math.sin(angle * FP.RAD);
                
				Draw.linePlus(startx, starty, endx, endy, color, 1.0, thickness);
				//Draw.linePlus(startx, starty, endx, endy, 0xFFFFFF, 1.0, 1.0); //debug
				
				startx = endx;
				starty = endy;
				
            }
			
			/*
			trace(startx);
			trace(starty);
			*/
        }
		
		override public function render():void 
		{
			
			//background art
			
			Draw.rect(FP.camera.x, FP.camera.y, 800, 600, 0x000033);
			
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, 300, BackgroundArcPos, BackgroundArcPos + 40, 0.1, 0x000055, 30);
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, 300, BackgroundArcPos + 180, BackgroundArcPos + 220, 0.1, 0x000055, 30);
			
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, (Math.sin(BackgroundArcPos / 20) * FP.DEG) + 200, BackgroundArcPos * 2, BackgroundArcPos * 2 + 80, 0.1, 0x000077, 15);
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, (Math.sin(BackgroundArcPos / 20) * FP.DEG) + 200, BackgroundArcPos * 2 + 180, BackgroundArcPos * 2 + 260, 0.1, 0x000077, 15);
			
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, 450, BackgroundArcPos * 2, BackgroundArcPos * 2 + 80, 0.1, 0x000044, 150);
			draw_arc(FP.camera.x + 400, FP.camera.y + 300, 450, BackgroundArcPos * 2 + 180, BackgroundArcPos * 2 + 260, 0.1, 0x000044, 150);
			
			//connections
			Draw.curve(400, 50, 400, -400, 750, -400, 0x333333, 1.0, 80);
			
			Draw.curve(350, 50, 350, -450, 750, -450, 0xFFFFFF, 1.0, 20);
			Draw.curve(450, 50, 450, -350, 750, -350, 0xFFFFFF, 1.0, 20);
			
			Draw.curve(1250, -400, 1500, -400, 1500, -650, 0x333333, 1.0, 80);
			
			Draw.curve(1250, -450, 1450, -450, 1450, -650, 0xFFFFFF, 1.0, 20);
			Draw.curve(1250, -350, 1550, -350, 1550, -650, 0xFFFFFF, 1.0, 20);
			
			Draw.curve(1250, -900, 1100, -900, 1100, -810, 0x333333, 1.0, 60);
			
			Draw.curve(1250, -930, 1075, -930, 1075, -810, 0xFFFFFF, 1.0, 5);
			Draw.curve(1250, -870, 1125, -870, 1125, -810, 0xFFFFFF, 1.0, 5);
			
			Draw.linePlus(1052.5, -747.5, 795, -747.5, 0x333333, 1.0, 80);
			
			Draw.linePlus(1052.5, -790, 795, -790, 0xFFFFFF, 1.0, 5);
			Draw.linePlus(1052.5, -710, 795, -710, 0xFFFFFF, 1.0, 5);
			
			Draw.linePlus(552.5, -747.5, -1000, -747.5, 0x333333, 1.0, 80);
			
			Draw.linePlus(552.5, -790, -1000, -790, 0xFFFFFF, 1.0, 5);
			Draw.linePlus(552.5, -710, -1000, -710, 0xFFFFFF, 1.0, 5);
			
			//platforms
			
			for each (var C : CircleWorld in CircleArray)
			{
				
				Draw.circlePlus(C.x, C.y, C.radius, 0x333333, 1.0, true);
				
				//debug
				//Draw.circlePlus(C.x, C.y, 10, 0xFFFFFF, 1.0, true);
				
				var PlatformArray : Array = C.GetPlatformArray();
				
				for each (var P : Platform in PlatformArray)
				{
					draw_arc(C.x, C.y, C.radius - (C.radius * P.y), P.x, P.x + P.platwidth, 0.2, P.platcolor, C.radius * P.platheight);
				}
				
			}

			if (GamePlayer.GetMovingBetweenWorlds() || EndGameTriggered)
			{
				Draw.circlePlus(GamePlayer.x, GamePlayer.y, 15, 0xAAAAAA, 1.0, true);
			}
			else
			{
				//player
				draw_arc(CircleArray[CurrentCircle-1].x, CircleArray[CurrentCircle-1].y, CircleArray[CurrentCircle-1].radius - (CircleArray[CurrentCircle-1].radius  * GamePlayer.y), GamePlayer.x - 1, GamePlayer.x + 1, 1.0, 0xAAAAAA, CircleArray[CurrentCircle-1].radius  * 0.1);
			}
			
			super.render();
		}
		
	}

}